@@ -124,6 +124,7 @@
 #include "config.h"
 
 #include <string.h>
+#include <fnmatch.h>
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n-lib.h>
@@ -257,6 +258,7 @@ static void totem_pl_parser_get_property (GObject *object,
 struct TotemPlParserPrivate {
 	GHashTable *ignore_schemes; /* key = char *, value = boolean */
 	GHashTable *ignore_mimetypes; /*key = char *, value = boolean */
+	GHashTable *ignore_globs; /*key = char *, value = boolean */
 	GMutex ignore_mutex;
 	GThread *main_thread; /* see CALL_ASYNC() in *-private.h */
 
@@ -325,8 +327,6 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 
 	totem_pl_parser_parent_class = g_type_class_peek_parent (klass);
 
-	g_type_class_add_private (klass, sizeof (TotemPlParserPrivate));
-
 	object_class->finalize = totem_pl_parser_finalize;
 	object_class->set_property = totem_pl_parser_set_property;
 	object_class->get_property = totem_pl_parser_get_property;
@@ -1136,8 +1136,8 @@ totem_pl_parser_save (TotemPlParser      *parser,
                       TotemPlParserType   type,
                       GError            **error)
 {
-        g_return_val_if_fail (TOTEM_IS_PL_PARSER (parser), FALSE);
-        g_return_val_if_fail (TOTEM_IS_PL_PLAYLIST (playlist), FALSE);
+        g_return_val_if_fail (TOTEM_PL_IS_PARSER (parser), FALSE);
+        g_return_val_if_fail (TOTEM_PL_IS_PLAYLIST (playlist), FALSE);
         g_return_val_if_fail (G_IS_FILE (dest), FALSE);
 
         if (totem_pl_playlist_size (playlist) == 0) {
@@ -1276,25 +1276,25 @@ totem_pl_parser_read_ini_line_string (char **lines, const char *key)
 static void
 totem_pl_parser_init (TotemPlParser *parser)
 {
-	parser->priv = G_TYPE_INSTANCE_GET_PRIVATE (parser, TOTEM_TYPE_PL_PARSER, TotemPlParserPrivate);
+	parser->priv = g_new0 (TotemPlParserPrivate, 1);
 	parser->priv->main_thread = g_thread_self ();
 	g_mutex_init (&parser->priv->ignore_mutex);
 	parser->priv->ignore_schemes = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 	parser->priv->ignore_mimetypes = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+	parser->priv->ignore_globs = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 }
 
 static void
 totem_pl_parser_finalize (GObject *object)
 {
-	TotemPlParserPrivate *priv = TOTEM_PL_PARSER (object)->priv;
-
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (priv != NULL);
+	TotemPlParser *parser = TOTEM_PL_PARSER (object);
+	TotemPlParserPrivate *priv = parser->priv;
 
 	g_clear_pointer (&priv->ignore_schemes, g_hash_table_destroy);
 	g_clear_pointer (&priv->ignore_mimetypes, g_hash_table_destroy);
-
+	g_clear_pointer (&priv->ignore_globs, g_hash_table_destroy);
 	g_mutex_clear (&priv->ignore_mutex);
+	g_clear_pointer (&parser->priv, g_free);
 
 	G_OBJECT_CLASS (totem_pl_parser_parent_class)->finalize (object);
 }
@@ -1584,6 +1584,28 @@ totem_pl_parser_mimetype_is_ignored (TotemPlParser *parser,
 	return ret;
 }
 
+static gboolean
+totem_pl_parser_glob_is_ignored (TotemPlParser *parser,
+				 const char *filename)
+{
+	GHashTableIter iter;
+	gpointer key;
+	int ret;
+
+	g_mutex_lock (&parser->priv->ignore_mutex);
+	g_hash_table_iter_init (&iter, parser->priv->ignore_globs);
+	while (g_hash_table_iter_next (&iter, &key, NULL)) {
+		const char *glob = key;
+
+		ret = fnmatch (glob, filename, 0);
+		if (ret == 0)
+			break;
+	}
+	g_mutex_unlock (&parser->priv->ignore_mutex);
+
+	return (ret == 0);
+}
+
 /**
  * totem_pl_parser_ignore:
  * @parser: a #TotemPlParser
@@ -1604,40 +1626,32 @@ totem_pl_parser_mimetype_is_ignored (TotemPlParser *parser,
 gboolean
 totem_pl_parser_ignore (TotemPlParser *parser, const char *uri)
 {
-	char *mimetype;
-	GFile *file;
+	g_autofree char *mimetype;
+	g_autoptr(GFile) file;
 	guint i;
 
+	if (totem_pl_parser_glob_is_ignored (parser, uri) != FALSE)
+		return TRUE;
+
 	file = g_file_new_for_path (uri);
-	if (totem_pl_parser_scheme_is_ignored (parser, file) != FALSE) {
-		g_object_unref (file);
+	if (totem_pl_parser_scheme_is_ignored (parser, file) != FALSE)
 		return TRUE;
-	}
-	g_object_unref (file);
 
 	//FIXME wrong for win32
 	mimetype = g_content_type_guess (uri, NULL, 0, NULL);
-	if (mimetype == NULL || strcmp (mimetype, UNKNOWN_TYPE) == 0) {
-		g_free (mimetype);
+	if (mimetype == NULL || strcmp (mimetype, UNKNOWN_TYPE) == 0)
 		return FALSE;
-	}
 
 	for (i = 0; i < G_N_ELEMENTS (special_types); i++) {
-		if (strcmp (special_types[i].mimetype, mimetype) == 0) {
-			g_free (mimetype);
+		if (strcmp (special_types[i].mimetype, mimetype) == 0)
 			return FALSE;
-		}
 	}
 
 	for (i = 0; i < G_N_ELEMENTS (dual_types); i++) {
-		if (strcmp (dual_types[i].mimetype, mimetype) == 0) {
-			g_free (mimetype);
+		if (strcmp (dual_types[i].mimetype, mimetype) == 0)
 			return FALSE;
-		}
 	}
 
-	g_free (mimetype);
-
 	return TRUE;
 }
 
@@ -1780,9 +1794,10 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 				GFile *base_file,
 				TotemPlParseData *parse_data)
 {
-	char *mimetype;
+	g_autofree char *mimetype = NULL;
+	g_autofree gpointer data = NULL;
+	g_autofree char *uri = NULL;
 	guint i;
-	gpointer data = NULL;
 	TotemPlParserResult ret = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 	gboolean found = FALSE;
 
@@ -1820,18 +1835,21 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 	if (!parse_data->recurse && parse_data->recurse_level > 0)
 		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
 
+	uri = g_file_get_uri (file);
+
 	/* Should we try to parse it with quvi? */
-	if (g_file_has_uri_scheme (file, "http")) {
-		char *url;
-		url = g_file_get_uri (file);
-		if (url != NULL && totem_pl_parser_is_videosite (url, parser->priv->debug) != FALSE) {
+	if (g_file_has_uri_scheme (file, "http") ||
+	    g_file_has_uri_scheme (file, "https")) {
+		if (uri != NULL && totem_pl_parser_is_videosite (uri, parser->priv->debug) != FALSE) {
 			ret = totem_pl_parser_add_videosite (parser, file, base_file, parse_data, NULL);
-			if (ret == TOTEM_PL_PARSER_RESULT_SUCCESS) {
-				g_free (url);
+			if (ret == TOTEM_PL_PARSER_RESULT_SUCCESS)
 				return ret;
-			}
 		}
-		g_free (url);
+	}
+
+	if (uri != NULL) {
+		if (totem_pl_parser_glob_is_ignored (parser, uri))
+			return TOTEM_PL_PARSER_RESULT_IGNORED;
 	}
 
 	/* In force mode we want to get the data */
@@ -1865,13 +1883,13 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 	/* Not a directory on http though */
 	if (g_strcmp0 (mimetype, "inode/directory") == 0 &&
 	    g_file_has_uri_scheme (file, "http")) {
-		g_free (mimetype);
-		mimetype = NULL;
+		g_clear_pointer (&mimetype, g_free);
 	}
 
 	DEBUG(file, g_print ("_get_mime_type_for_name for '%s' returned '%s'\n", uri, mimetype));
-	if (mimetype == NULL || strcmp (UNKNOWN_TYPE, mimetype) == 0
-	    || (g_file_is_native (file) && g_content_type_is_a (mimetype, "text/plain") != FALSE)) {
+	if (mimetype == NULL ||
+	    strcmp (UNKNOWN_TYPE, mimetype) == 0 ||
+	    g_content_type_is_a (mimetype, "text/plain") != FALSE) {
 		char *new_mimetype;
 		new_mimetype = my_g_file_info_get_mime_type_with_data (file, &data, parser);
 		if (new_mimetype) {
@@ -1883,20 +1901,13 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 		}
 	}
 
-	if (mimetype == NULL) {
-		g_free (data);
+	if (mimetype == NULL)
 		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
-	}
 
-	if (strcmp (mimetype, EMPTY_FILE_TYPE) == 0) {
-		g_free (data);
-		g_free (mimetype);
+	if (strcmp (mimetype, EMPTY_FILE_TYPE) == 0)
 		return TOTEM_PL_PARSER_RESULT_SUCCESS;
-	} else if (strcmp (mimetype, HLS_MIME_TYPE) == 0) {
-		g_free (data);
-		g_free (mimetype);
+	else if (strcmp (mimetype, HLS_MIME_TYPE) == 0)
 		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
-	}
 
 	/* If we're at the top-level of the parsing, try to get more
 	 * data from the playlist parser */
@@ -1910,11 +1921,8 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 		DEBUG(file, g_print ("_get_mime_type_with_data for '%s' returned '%s' (was %s)\n", uri, mimetype, AUDIO_MPEG_TYPE));
 	}
 
-	if (totem_pl_parser_mimetype_is_ignored (parser, mimetype) != FALSE) {
-		g_free (mimetype);
-		g_free (data);
+	if (totem_pl_parser_mimetype_is_ignored (parser, mimetype) != FALSE)
 		return TOTEM_PL_PARSER_RESULT_IGNORED;
-	}
 
 	if (parse_data->recurse || parse_data->recurse_level == 0) {
 		parse_data->recurse_level++;
@@ -1924,8 +1932,6 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 				DEBUG(file, g_print ("URI '%s' is special type '%s'\n", uri, mimetype));
 				if (parse_data->disable_unsafe != FALSE && special_types[i].unsafe != FALSE) {
 					DEBUG(file, g_print ("URI '%s' is unsafe so was ignored\n", uri));
-					g_free (mimetype);
-					g_free (data);
 					return TOTEM_PL_PARSER_RESULT_IGNORED;
 				}
 				if (base_file == NULL)
@@ -1969,8 +1975,7 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 				if ((func == NULL && mimetype != NULL) || (mimetype == NULL && dual_types[i].func == NULL)) {
 					DEBUG(file, g_print ("Ignoring URI '%s' because we couldn't find a playlist parser for '%s'\n", uri, mimetype));
 					ret = TOTEM_PL_PARSER_RESULT_UNHANDLED;
-					g_free (mimetype);
-					mimetype = NULL;
+					g_clear_pointer (&mimetype, g_free);
 					break;
 				} else if (func == NULL) {
 					func = dual_types[i].func;
@@ -1991,21 +1996,14 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 			}
 		}
 
-		g_free (data);
-
 		parse_data->recurse_level--;
 	}
 
-	if (ret == TOTEM_PL_PARSER_RESULT_SUCCESS) {
-		g_free (mimetype);
+	if (ret == TOTEM_PL_PARSER_RESULT_SUCCESS)
 		return ret;
-	}
 
-	if (totem_pl_parser_ignore_from_mimetype (parser, mimetype) != FALSE) {
-		g_free (mimetype);
+	if (totem_pl_parser_ignore_from_mimetype (parser, mimetype) != FALSE)
 		return TOTEM_PL_PARSER_RESULT_IGNORED;
-	}
-	g_free (mimetype);
 
 	if (ret != TOTEM_PL_PARSER_RESULT_SUCCESS && parse_data->fallback) {
 		totem_pl_parser_add_one_file (parser, file, NULL);
@@ -2074,7 +2072,7 @@ totem_pl_parser_parse_with_base_async (TotemPlParser *parser, const char *uri, c
 	GTask *task;
 	ParseAsyncData *data;
 
-	g_return_if_fail (TOTEM_IS_PL_PARSER (parser));
+	g_return_if_fail (TOTEM_PL_IS_PARSER (parser));
 	g_return_if_fail (uri != NULL);
 	g_return_if_fail (strstr (uri, "://") != NULL);
 
@@ -2110,7 +2108,7 @@ totem_pl_parser_parse_with_base (TotemPlParser *parser, const char *uri,
 	TotemPlParserResult retval;
 	TotemPlParseData data;
 
-	g_return_val_if_fail (TOTEM_IS_PL_PARSER (parser), TOTEM_PL_PARSER_RESULT_UNHANDLED);
+	g_return_val_if_fail (TOTEM_PL_IS_PARSER (parser), TOTEM_PL_PARSER_RESULT_UNHANDLED);
 	g_return_val_if_fail (uri != NULL, TOTEM_PL_PARSER_RESULT_UNHANDLED);
 	g_return_val_if_fail (strstr (uri, "://") != NULL,
 			TOTEM_PL_PARSER_RESULT_ERROR);
@@ -2185,7 +2183,7 @@ totem_pl_parser_parse_finish (TotemPlParser *parser, GAsyncResult *async_result,
 {
 	GTask *task = G_TASK (async_result);
 
-	g_return_val_if_fail (TOTEM_IS_PL_PARSER (parser), FALSE);
+	g_return_val_if_fail (TOTEM_PL_IS_PARSER (parser), FALSE);
 	g_return_val_if_fail (g_task_is_valid (async_result, parser), FALSE);
 
 	/* Propagate any errors which were caught and return the result; otherwise just return the result */
@@ -2228,7 +2226,7 @@ totem_pl_parser_add_ignored_scheme (TotemPlParser *parser,
 {
 	char *s;
 
-	g_return_if_fail (TOTEM_IS_PL_PARSER (parser));
+	g_return_if_fail (TOTEM_PL_IS_PARSER (parser));
 
 	g_mutex_lock (&parser->priv->ignore_mutex);
 
@@ -2252,13 +2250,34 @@ void
 totem_pl_parser_add_ignored_mimetype (TotemPlParser *parser,
 		const char *mimetype)
 {
-	g_return_if_fail (TOTEM_IS_PL_PARSER (parser));
+	g_return_if_fail (TOTEM_PL_IS_PARSER (parser));
 
 	g_mutex_lock (&parser->priv->ignore_mutex);
 	g_hash_table_insert (parser->priv->ignore_mimetypes, g_strdup (mimetype), GINT_TO_POINTER (1));
 	g_mutex_unlock (&parser->priv->ignore_mutex);
 }
 
+/**
+ * totem_pl_parser_add_ignored_glob:
+ * @parser: a #TotemPlParser
+ * @glob: a glob to ignore
+ *
+ * Adds a glob to the list of mimetypes to ignore, so that
+ * any URI of that glob is ignored during playlist parsing.
+ *
+ * Since: 3.26.4
+ **/
+void
+totem_pl_parser_add_ignored_glob (TotemPlParser *parser,
+				  const char    *glob)
+{
+	g_return_if_fail (TOTEM_PL_IS_PARSER (parser));
+
+	g_mutex_lock (&parser->priv->ignore_mutex);
+	g_hash_table_insert (parser->priv->ignore_globs, g_strdup (glob), GINT_TO_POINTER (1));
+	g_mutex_unlock (&parser->priv->ignore_mutex);
+}
+
 /**
  * totem_pl_parser_parse_duration:
  * @duration: the duration string to parse
@@ -2342,27 +2361,24 @@ totem_pl_parser_parse_duration (const char *duration, gboolean debug)
 guint64
 totem_pl_parser_parse_date (const char *date_str, gboolean debug)
 {
-	GTimeVal val;
+	g_autoptr(GDateTime) date = NULL;
 
 	g_return_val_if_fail (date_str != NULL, -1);
 
-	memset (&val, 0, sizeof(val));
 	/* Try to parse as an ISO8601/RFC3339 date */
-	if (g_time_val_from_iso8601 (date_str, &val) != FALSE) {
+	date = g_date_time_new_from_iso8601 (date_str, NULL);
+	if (date != NULL) {
 		D(g_message ("Parsed duration '%s' using the ISO8601 parser", date_str));
-		return val.tv_sec;
+		return g_date_time_to_unix (date);
 	}
 	D(g_message ("Failed to parse duration '%s' using the ISO8601 parser", date_str));
 	/* Fall back to RFC 2822 date parsing */
-	{
-		g_autoptr(GDateTime) date = NULL;
-		date = g_mime_utils_header_decode_date (date_str);
-		if (!date || !g_date_time_to_timeval (date, &val)) {
-			D(g_message ("Failed to parse duration '%s' using the RFC 2822 parser", date_str));
-			return -1;
-		}
-		return val.tv_sec;
+	date = g_mime_utils_header_decode_date (date_str);
+	if (!date) {
+		D(g_message ("Failed to parse duration '%s' using the RFC 2822 parser", date_str));
+		return -1;
 	}
+	return g_date_time_to_unix (date);
 }
 #endif /* !TOTEM_PL_PARSER_MINI */
 
